EMregion<-function(spect,par0,pro0,indMode,tol=10^(-5),alpha=0.995,maxiter=100,sOpt="fix",warning.path,cluster.id,spect.id){
  
  ##Purpose: Does the EM algorithm as suggested by the paper Dijkstra
  ##Input
  ##  -spect: a matrix where the first column is the mz and the second column is the intensity.
  ##  -par0: a matrix where the first column is the shift parameter, second column is the mu parameter,
  ##         and the third column is the sigma parameter for the shifted lognormal distribution.
  ##         The number of rows corresponding on the number of mode in the region.
  ##  -pro0: a vector of length corresponding to the number of modes.
  ##  -indMode: if the string is "mz", it will use mz values instead of the indices
  ##  -tol: the tolerence level for which if the euclidean distance is less tol, there is convergence
  ##        It is also used to determine the smalled
  ##  -maxiter: the max number of iteration.
  ##  -sOpt: the option for the parameter s. It is either fixed for all iterations or varying/updated.
  ##  -alpha: a scalar variable which is used if sOpt!="fix". It determines how to update the parameter s.
  ##  -warning.path: a string indicating where the warning path is to be printed
  ##  -cluster.id: a numeric indicating the cluster id
  ##  -spect.id: a numeric indicating the spectrum id
  ##
  ##==Comments: 050107, included 3 new arguements, warning.path, cluster.id, spect.id.
##browser()
  ##The number of peaks
  M<-nrow(par0)
  
  ##The indice values are used to define the mz dimension
  ## Using the mz values
  if(indMode=="mz"){
    y<-spect[,1]
  }else{
    y<-1:nrow(spect)
  }

  ##The profile/counts for each entry of y
  N<-spect[,2]

  ##...The expectation step with initial estimates
  ##Initial estimates of shift, mean and sigma parameter
  para0<-par0
  ##Initial estimates of the proportions
  p0<-pro0
  ##Initial convergence status and iteration no
  noConverge<-TRUE;noiter<-0
  
  while(noConverge & noiter<maxiter){
   
    ##...The expectation step with initial/previous estimates
    ##The indivdual components of the mixture model (4)
    pf<-apply(matrix(1:M,ncol=1),1,"mix",
              y=y,p=p0,para=para0)
    ##The finite mixture model in (4)
    f<-apply(pf,1,sum)
    ##Calculate the conditional component memebership probabilities
    pji<-t(scale(t(pf),center=FALSE,scale =f))

    ##...The maximization step
    if(sOpt=="fix"){
      Mout<-Mstep(y,N,pji,para0)
    }else{
      Mout<-Mstep(y,N,pji,para0,alpha)
    }
    
    ##The updated parameters
    para1<-Mout[[1]]
    p1<-Mout[[2]]
    
    ##Determine if the parameters converge
    if(sOpt=="fix"){
      updatePara<-c(para1[,2],para1[1,3],p1[-length(p1)])
      oldPara<-c(para0[,2],para0[1,3],p0[-length(p0)])
    }else{
      updatePara<-c(para1[,1],para1[,2],para1[1,3],p1[-length(p1)])
      oldPara<-c(para0[,1],para0[,2],para0[1,3],p0[-length(p0)])
    }
    
    noConverge<-((sum((updatePara-oldPara)^2)/length(oldPara))>=tol)

    ##Update the old parameters with new parameters
    para0<-para1
    p0<-p1
    ##Update the no. of iterations
    noiter<-noiter+1
    print(noiter)
    
  }

  if(noConverge==TRUE){
    
    ##Output a warning to the file
    write(paste("No convergence for EM aglorithm at cluster.id:",cluster.id,"spectrum.id:",spect.id),
          file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
    print(paste("No convergence for EM aglorithm at cluster.id:",cluster.id,"spectrum.id:",spect.id))
    
  }

  ##Return the converged parameters
  return(list(para1,p1,converge=!noConverge,iter=noiter))
}
