Mstep<-function(y,N,pji,para0,alpha=NULL){
  
  ##Purpose: The M step to update the parameters.
  ##         Sub-routine for EMregion
  ##=Input:
  ##  -y is a vector of the mz/indice of the spectrum
  ##  -N is a vector of the profile/density/count of the spectrum
  ##  -pji is a matrix of conditional component membership probabilities.
  ##   The rows corresponds to the mz observations while the columns are the components
  ##  -para0: a matrix where the first column is the shift parameter, second column is the mu parameter,
  ##          and the third column is the sigma parameter for the shifted lognormal distribution.
  ##          The number of rows corresponding on the number of mode in the region.
  ##  -alpha: The scalar used to estimate the parameter s.
  ##
  ##=Output:
  ##  -a list where
  ##   [[1]]
  ##   a matrix with 1st col shift parameter, 2nd col mean and 3rd col sigma (j<=M);
  ##   the rows correspond to the no. of modes  while the attr(para,"rate") is the
  ##   rate parameter of the exponential distribution
  ##   [[2]]
  ##   a vector consisting of the probabilities for each component

  ##Extract the parameters
  s<-para0[,1]
  M<-nrow(para0)

  ##To store the updated mu for each component
  mu<-NULL
  ##To store the components to compute the updated sigma
  sigma2Num<-NULL
  sigma2Den<-NULL
  ##To store the updated p for each component
  p<-NULL
  
  for(j in 1:M){
    
    ##Subtract the shift from the time values and take the log of the positive part of the outcome
    ##Identify those with positive difference
    valid<-which(y>s[j])
    y2<-rep(0,times=length(y))
    y2[valid]<-log(y[valid]-s[j])
    
    ##The updated mu; see (6)
    mu<-c(mu,sum(N[valid]*pji[valid,j]*y2[valid])/sum(N[valid]*pji[valid,j]))
    
    ##The numerator of the updated sigma^2
    sigma2Num<-c(sigma2Num,sum(N[valid]*pji[valid,j]*(y2[valid]-mu[j])^2))
    ##The denominator of the updated sigma^2
    sigma2Den<-c(sigma2Den,sum(N[valid]*pji[valid,j]))

    ##The updated proportion; see (9)
    p<-c(p,sum(N[valid]*pji[valid,j])/sum(N))

  }
  
  ##The updated sigma^2; see (7)
  sigma2<-sum(sigma2Num)/sum(sigma2Den)

  ##The updated proportion; see (9)
  p<-c(p[-M],(1-sum(p[-M])))
  
  ##Consolidating the updated parameters
  if(is.null(alpha)){##Do not update the parameter s
    para<-cbind(s,mu,sigma=rep(sqrt(sigma2),M))
  }else{##Update the parameter s
    ##See (16) or (17) on the equation
    para<-cbind(s=alpha*(exp(mu-sigma2)+s),mu,sigma=rep(sqrt(sigma2),M))
  }
  
  ##The updated parameters from the M-step
  out<-list(para,p)

  return(out)
  
}
