"alignOptimMultiPeak" <-
function(refSpect,idOth,data.default,plot.indicator=FALSE,
         warning.path,cluster.id,peak.id){
  ##refSpect: A list containing information about the reference spectrum and the peak range.
  ##idOth: the id of the other spectra.
  ##data.default: a array of mz and intensity values

  
  ##row.id: The row.id of the region which is the union of row.ids of a peak.
  row.id<-refSpect$row.idPeakPCA
  ##excessPt
  excessPt<-refSpect$excessPt
  ##The values to make the intensity values positive
  min.int<-refSpect$min.int
  ##The intensity values of the reference spectrum
  Ref<-refSpect$PC1
  ##The minimum and maximum mz of the peak region
  RefMin.mz<-refSpect$peakRange[1]
  RefMax.mz<-refSpect$peakRange[2]
  ##The cluster of the peak region
  data.cluster<-data.default[row.id,,]
  
  ##==Include attributes to the vector of the template
  ##The row indice of the maximum point of the template peak
  RefMaxPos<-refSpect$peakValley[2,"id"]
  ##The intensity of the max point (amplitude) of the template peak
  maxAmp<-refSpect$peakValley[2,"intensity"]
  ##The row indice of within the valid template
  tmp<-refSpect$peakValley[,"id"]
  tmp<-tmp[!is.na(tmp)]
  idTemplate<-range(tmp)

  ##================================##
  ##Setting up the optimization step##
  ##================================##
    
  ##=========================================================##
  ##==Identify those template with only 1 peak and 1 valley==##
  ##=========================================================##
  if(any(is.na(refSpect$peakValley[,"id"]))){

    ##===Use the local max and min to fit the data.

    ##...Print the warning messages
    write(paste("Template does not have proper peak with valleys","cluster.id:",cluster.id,
                "peak.id:",peak.id,"idOth:",idOth,"Not fitted to template."),
          file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
    
    print(paste("warning: The other spectrum (id=",idOth,") is not fitted to template spectrum's peak (id",
                peak.id,") in cluster (id= ",cluster.id,") which does not have proper valleys.",sep=""))

    ##...Determine the points in which the spectrum is within the peak range
    ##Indices within the mz range
    valid<-which(data.cluster[,idOth,1]>=RefMin.mz & data.cluster[,idOth,1]<=RefMax.mz)
    ##Indice of the max point for the spectrum
    Oth.max<-which.max(data.cluster[valid,idOth,2])
    ##Indice of the left valley for the spectrum
    maxLeft<-which.min(data.cluster[valid[1:Oth.max],idOth,2])
    ##Indice of the right valley for the spectrum
    maxRight<-which.min(data.cluster[valid[Oth.max:length(valid)],idOth,2])
    
    out<-c(Oth.max,(data.cluster[valid[Oth.max],idOth,1]),
           (data.cluster[valid[Oth.max],idOth,2]-min.int[idOth]),
           NA,NA,NA,NA,
           (row.id[1]+valid[maxLeft]-1),data.cluster[valid[maxLeft],idOth,1],
           (row.id[1]+valid[Oth.max:length(valid)][maxRight]-1),
           data.cluster[valid[Oth.max:length(valid)][maxRight],idOth,1],NA)
    
    names(out)<-c("max","max.mz","max.ampMix","step","scaleMix","mseMix","df",
                  "leftValley","left.mz","rightValley","right.mz","excessPt")
    
    return(out)
    
  }


  ##======================================================================================##
  ##==The max positive steps and negative steps that this other spectrum can take       ==##
  ##==given the peak of the template spectrum cannot cross over the other spectrum edges==##
  ##==NOTE: It is possible that both the negative and positive steps are 0. This is     ==##
  ##==      because the the other spectrum may have its mz values misalign with the     ==##
  ##==      template.                                                                   ==##
  ##======================================================================================##
  
  ##==========================================================================================================##
  ##==Determine the max number of steps needed (Using the right of the other spectrum as the starting point)==##
  ##==========================================================================================================##
  ##To find the number of postive steps is the number of points between the left valley and max point
  stepNegative<--(RefMaxPos-idTemplate[1])
  ##To find the number of negative steps is the number of points between the right valley and max point
  stepPositive<-(length(Ref)-RefMaxPos)-(excessPt[1,idOth]+excessPt[2,idOth])
  
  step.tmp<-stepNegative:stepPositive
      
  ##=============================================================================##
  ##Using apply() to find the optimial scale value for each step from grid vector##
  ##=============================================================================##
  ##A function that optimize the objective function with each fixed step and allowing scale to vary
  stepFunction<-function(step,Ref,idOth,data.default,row.id,min.int,idTemplate,excessPt,
                         warning.path,cluster.id,peak.id){
    ##Argument
    ##-step: is a scalar indicating the number of steps
    ##-Ref: is a vector of the template spectrum's intensity values
    ##-idOth: is a scalar indicating the id (or col no) of the other spectrum
    ##-data.default: is a proteomic array of the cluster
    ##-row.id: is a vector of the row indices of data.default corresponding to the peak region of interest.
    ##-min.int: is a vector corresponding to the min intensity to make the region positive.
    ##-excessPt: matrix containing the number of excess points on the left side (top row) and right side (2nd row).
    ##-warning.path: is a string indicating the path to save the warning files/
    ##-cluster.id: is a scalar indicating the id of the cluster.
    ##-peak.id: is a scalar indicating the id of the template spectrum's peak.
    ##Output
    ##-A vector consisting of the mean square error, its df, its scale and OthMin 

      
    ##...The objective function: Scaled MSE and the scale parameter is estimated via area under the curve
    ObjFun<-function(Ref,Oth){
      
      ##Argument
      ##-Ref: is a vector of the template spectrum intensity value
      ##-Oth: is a vector of the other spectrum intensity value
      ##-scale: is a scalar corresponding to scale parameter
      
      ##...Compute the mean square error, scaled to by the mean value of the other spectrum
      ##The weight applied to the MSE
      wt<-quantile(Oth, probs = seq(0.5, 0.5, length.out=1))
      ##The scale estimated via area under the curve
      scaleMix<-sum(Oth)/sum(Ref)
      ##The objective function
      mse<-(sum((Oth-scaleMix*Ref)^2)/length(Ref))/(wt^2)
      
      return(list(MSE=mse,scale=scaleMix))
    }

    ##===========================================##
    ##==This step implements the step parameter==##
    ##===========================================##
    ##The intensity value of the other spectrum
    Oth<-data.default[,idOth,2]
    ##The min value of the intensity to make the peak region of interest positive
    min.intOth<-min.int[idOth]
    ##Get the spectra aligned with the left side of other spectrum within the valid mz range as a starting point
    valid<-(row.id+step+excessPt[1,idOth])[1:(diff(idTemplate)+1)]
    ##Check if the valid is out of the window indices
    if(any(valid<1 | valid>nrow(data.default))){
      out<-c(Inf,Inf,Inf)
      names(out)<-c("mseMix","df","scaleMix")
      return(out)
    }
    
    ##Get the region of interest
    Oth<-Oth[valid]-min.intOth
    ##Make the negative intensity positive
    Oth[Oth<0]<-10^(-8)
    
    ##The intensity value of the template
    Ref<-Ref[idTemplate[1]:idTemplate[2]]

    ##Output of the optimized function to be returned
    output<-ObjFun(Ref,Oth)
    
    ##The information to be given out by the function stepFunction
    df<-length(valid)
    out<-c(output$MSE,df,output$scale)
    names(out)<-c("mseMix","df","scaleMix")
    return(out)
  }
  
  ##===================================================================##
  ##==Obtain the optimal scale and step with its relevent information==##
  ##===================================================================##
  
  ##The restricted number of steps that can be taken
  step.tmp<-unique(step.tmp)
  
  ##The consisting of the scaled MSE and 
  obj.tmp<-apply(matrix(step.tmp,ncol=1),1,"stepFunction",Ref=Ref,idOth=idOth,
                 data.default=data.default,row.id=row.id,min.int=min.int,idTemplate=idTemplate,excessPt=excessPt,
                 warning.path=warning.path,cluster.id=cluster.id,peak.id=peak.id)
  ##Organizing the information of the different steps and the corresponding MSEs
  obj.tmp<-t(obj.tmp)
  rownames(obj.tmp)<-step.tmp
  ##Find the id of the objective function with the lowest value
  min.id<-which.min(obj.tmp[,"mseMix"])

  ##===================================================##
  ##Get the essential information from the optimization##
  ##===================================================##
  df<-obj.tmp[min.id,"df"]
  mseMix<-obj.tmp[min.id,"mseMix"]
  
  step<-step.tmp[min.id]
  scaleMix<-obj.tmp[min.id,"scaleMix"]
    
  ##==========================================================##
  ##==To plot the objective function with scale against step==##
  ##==========================================================##
  if(plot.indicator==TRUE){
    plot(step.tmp,obj.tmp[,"MSE"],ylab="objective function",xlab="step")
    abline(v=step.tmp[min.id])
    oldpar<-par(new=TRUE)
    plot(step.tmp,obj.tmp[,"scaleMix"],type="l",col="red",xlab="",ylab="",axes=FALSE)
    axis(4)
    legend(min(step.tmp),min(obj.tmp[,2]),c("mseMix","scaleMix"),pch=c(19,0),lty=c(0,1),col=c("black","red"))
  } 
  
  ##==Identify the number of points the valleys are away from the peak of the template spectrum (valleyRef)
  ##==and then locate the valleys on the other spectrum (maxOth)
  ##The row indices for the other spectrum
  valid<-(row.id+step+excessPt[1,idOth])[1:(diff(idTemplate)+1)]
  ##The valleys in the template
  valleyRef<-refSpect$peakValley[c(1,3),"id"]
  
  ##==Locate the max on the other spectrum (maxOth)
  ##Identity the max point in the other spectrum
  maxOth<-valid[(RefMaxPos-valleyRef[1]+1)]
  ##To get intensity and m/z value of the peak of the other spectrum
  max.mz<-data.default[maxOth,idOth,1]
  max.ampMix<-maxAmp*scaleMix
  
  ##Identify the the row indices of the left valley and the mz values
  if(!is.na(valleyRef[1])){##Left valley exist
    leftValley<-valid[1]
    left.mz<-data.default[leftValley,idOth,1]
  }else{##Left valley does not exist
    leftValley<-NA
    left.mz<-NA
  }
  
  ##Identify the the row indices of the right valley and the mz values
  if(!is.na(valleyRef[2])){##Right valley exist
    rightValley<-valid[length(valid)]
    right.mz<-data.default[rightValley,idOth,1]
  }else{
    rightValley<-NA
    right.mz<-NA
  }

  ##The vector to be returned by the function
  output<-c(maxOth,max.mz,max.ampMix,step,scaleMix,mseMix,df,
            leftValley,left.mz,rightValley,right.mz,excessPt[1,idOth])
  names(output)<-c("max","max.mz","max.ampMix","step","scaleMix","mseMix","df",
                   "leftValley","left.mz","rightValley","right.mz","excessPt")
  return(output)
}

