###==Changes made on 23022007:
##   1. Noted that there are instances where it is possible to get first principal components
##      with both negative and positive values (i.e. U-shaped with the edges of the U-shape
##      having small negative values). If the negative values are small (i.e. < tol),
##      and the absolute value of the minimum is bigger than the absolute value of the maximun,
##      then we will set the negative values to zero.

##================================================##
##sub-function to perform constraint least-squares##
##================================================##
"lsfit.cont"<-
function(X,Y,Con){
  Dmat<-t(X)%*%X
  coef<-NULL
  
  if(is.matrix(Con)){##Multiple-peaks region requires restriction
    
    for(i in 1:ncol(Y)){
      dvec<-matrix(Y[,i],nrow=1)%*%X
      ##Non-negative contraint for the 1st column. The shift constraints at the 2nd and 3rd columns
      Amat<-matrix(c(1,0,-Con[1,i],-1,Con[2,i],1),ncol=3)
      obj<-solve.QP(Dmat,dvec,Amat,bvec=c(10^(-8),0,0))
      coef<-cbind(coef,c(obj$solution,identical(obj$solution,obj$unconstrainted.solution)))
    }
    
  }else{##Single-peak region does not require restriction
    
    ##Same contraint for all spectrum
    Amat<-matrix(c(1,0),ncol=1)
    for(i in 1:ncol(Y)){
      dvec<-matrix(Y[,i],nrow=1)%*%X
      obj<-solve.QP(Dmat,dvec,Amat)
      coef<-cbind(coef,c(obj$solution,identical(obj$solution,obj$unconstrainted.solution)))
    }
  }
  
  return(coef)
}

onePeakPCA<-function(dat,row.id,excessPt,tol=toler,maxit=maxiter,tolCon=tolConMain,mz.limit=mz.limit,symm.limit=symm.limit,
                     warning.path=warning.path,cluster.id=cluster.id){

  
  ##==========================================================================##
  ##Iterative Steps: Updating/Correcting the spectra with the estimated shifts##
  ##                 at each iteration.                                       ##
  ##==========================================================================##

  ##Store the percentage of variation explained by the PC
  amp.shift<-list(NULL)
  amp.shift[[1]]<-0
  ##MSE of the first and second PC
  MSE.PC1<-NULL;MSE.PC2<-NULL
  
  ##Keep track of the shift parameter
  shift.iter<-rep(0,ncol(dat))
  ##Initialize the iteration, converge and the updated/corrected intensities
  iter<-1; converge<-1;

  ##Extract the intensity values of the region of interest
  data.int<-dat[row.id,,2]

  ##==Make a shift up to make all intensity>=0
  ##Performing the non-negative shifting on the spectrum
  int.value<-apply(data.int,2,
                   function(x){
                     min.x<-min(x)
                     if(min.x<0){
                       return(x-min.x)
                     }else{
                       return(x)
                     }
                   })
  
  ##The min. value to make the intensities non-negative
  min.int<-apply(data.int,2,min)
  ##Set those intensities with originally non-negative intensity value to 0
  min.int[min.int>0]<-0

  while(iter<=maxit & all(converge>tol)){
    print(".")

    ##Peform SVD on the intensity of the cluster
    clust.svd<-svd(int.value)
    ##Get the percentage of the variability explained by each PC
    percent<-clust.svd$d^2/sum(clust.svd$d^2)
      
    ##==The empirical f' and f'' of the first principle component (f).
    ##f'
    d1<-diff(clust.svd$u[,1])
    ##Normalize f'
    d1<-d1/sqrt(sum(d1^2))
    ##f''
    d2<-diff(d1)
    ##Normalize f''
    d2<-d2/sqrt(sum(d2^2))
  
    ##==The first 2 principle component
    for(i in 1:2){
      if(i==1){##The first principle component
        
        ##Extract the first principle component
        P1<-clust.svd$u[,i]
        ##Make sure the zero term is not around
        P1<-P1[P1!=0]
        sign1<-sign(max(P1))

###===23022007(start): Made Changes Here

        ##If the max value is very small (i.e. <tol), and the magnitude of the smallest value > the largest value,
        ##and if all the values of the PC are not of the same sign.
        ##THEN we need to change the sign of sign1
        ##AND we need to set the negative values of P1 to 0 after multiplying it by the updated sign1
        if(abs(max(P1))<tol & abs(min(P1))>abs(max(P1)) & (!all(clust.svd$u[,i]>=0) & !all(clust.svd$u[,i]<=0))){
          ##Change the sign of sign1
          sign1<--1*sign1
          ##Identify those intensities that are negative after multiply sign1
          P1Neg<-which((sign1*clust.svd$u[,i])<=0)
          ##Replace those intensities that are negative as positive
          clust.svd$u[P1Neg,i]<-0
        }
                
        if((!(abs(max(P1))<tol) & abs(min(P1))>abs(max(P1)) & !all(clust.svd$u[,i]>=0) & !all(clust.svd$u[,i]<=0)) |
           all(clust.svd$u[,i]==0)){
          print(sum(clust.svd$u[,i]<=0)/length(clust.svd$u[,i]))
          print(range(clust.svd$u[,i]))
          plot(clust.svd$u[,i])
          browser()
          stop("There are some points in the PC1 that are positive and negative with max value not small enough. OR the PC1 are all zeros.")
        }

###===23022007 (End): Made Changes Here        
      }else{
        if(i==2){##The second principle component
          
          ##==To determine the best sign for the second PC by using the empirical first derivative
          ##The MSE between the minus of the empirical first derivative of PC1 and PC2
          minus.MSE<-sum((-sign1*d1-clust.svd$u[1:length(d1),i])^2)
          ##The MSE between the empirical first derivative of PC1 and PC2
          MSE<-sum((sign1*d1-clust.svd$u[1:length(d1),i])^2)
          if(MSE<=minus.MSE){##The positive PC2 is better than the negative PC2
            ##The positive PC2 is better
            sign2<-1
          }else{
            ##The negative PC2 is better
            sign2<--1
          }
        }
      }
    }
    
    ##The first principle component and second principle component
    pc<-cbind(sign1*clust.svd$u[,1],sign2*clust.svd$u[,2])

    ##==Getting the contraints for the shift
    ##========================================================##
    ##Update the contraints based on the template's peak shape##
    ##========================================================##
    shiftCon<-template.Info(data.default=dat,row.idPeakPCA=row.id,shift.iter=shift.iter,PC=pc,
                            mz.limit=mz.limit,symm.limit=symm.limit,
                            warning.path=warning.path,cluster.id=cluster.id,idRef="template within PCA")
    ##The 
    shiftCon<-matrix(rep(shiftCon*c(-1,1),
                         ncol(int.value)),ncol=ncol(int.value))
    
    ##==Consider the excess points, they reduce the number of shifting
    ##Adjust the left bound with the excess points taken into consideration
    leftCon<-shiftCon[1,]+excessPt[1,]
    ##Adjust the right bound with the excess points taken into consideration
    rightCon<-shiftCon[2,]-excessPt[2,]
    if(!all(leftCon<=rightCon)){
      stop("The constraints are not met!!!")
    }

    conAdj<-rbind(leftCon,rightCon)-rbind(shift.iter,shift.iter)

    ##Using contrainted estimation to get the beta estimates
    beta<-lsfit.cont(pc,int.value,conAdj)
    ##The shift is estimated by beta2/beta1
    beta[2,]<--beta[2,]/beta[1,]
    ##Just in case the scale/amplitude paramter (beta1) is 0.
    beta[2,]<-ifelse(is.infinite(beta[2,]) | is.nan(beta[2,]),0,beta[2,])
    ##Store the amplitude and shift parameter
    amp.shift[[iter]]<-beta

    ##Add up the total shift
    shift.iter<-shift.iter+beta[2,]

    ##Remove the scale shift only
    int.cor<-NULL
    for(i in 1:ncol(int.value)){
      
      ##Update the row.id with the shift
      row.idUpdate<-floor(row.id[1]+shift.iter[i]):ceiling(row.id[length(row.id)]+shift.iter[i])
      ##Which are the ids that are invalid
      invalid.id<-which(row.idUpdate<=0 | row.idUpdate>nrow(dat))
      
      if(length(invalid.id)>0){##The row indices are invalid
        
        ##Initialize the vector to record the intensity values
        tmp.int<-row.idUpdate
        ##Obtain the intensity values of ids in the valid region
        tmp.int[-invalid.id]<-dat[row.idUpdate[-invalid.id],i,2]-min.int[i]
        ##Intensity set to zero for those with invalid ids
        tmp.int[invalid.id]<-0
        
      }else{
        
        ##The intensities of those with the updated row.indices
        tmp.int<-dat[row.idUpdate,i,2]-min.int[i]
        
      }
      
      ##Those negative values are set to 0
      tmp.int[tmp.int<0]<-0
      
      ##The updated/corrected intensity
      int.cor<-cbind(int.cor,
                     approx(row.idUpdate,tmp.int,(row.id+shift.iter[i]),rule=2)$y)
    }
    
    ##Replace the intensity value (int.value) with the corrected one (int.cor)
    int.value<-int.cor
    
    ##Compute the convergence criterion: i.e. the sum of the squared difference between the two PC1s
    ##(i.e. present and previous iteration)
    if(iter==1){
      converge<-c(1,1);
    }else{
      converge<-c(per=abs(percent[1]-percent0[1]),pc=(sum((pc[,1]-pc0[,1])^2)/nrow(pc)))
    }
    
    ##Update the previous PCs to pc0
    pc0<-pc
    percent0<-percent
    ##Update the iteration
    iter<-iter+1
    
  }
  
  ##A list consisting the PC1, the shift parameter and the amplitude of the spectra w.r.t PC1
  output<-list(int.value=int.value,PC1=pc[,1],shift=shift.iter,amp=beta[1,],min.int=min.int,
               iter=iter,converge=converge)
  
  return(output)
  
}

