peakPA<-function(spect.id,collect.tmp,data.default,maxiter=100,toler=tol,indMode="mz",
                 sOpt="fix",warning.path,cluster.id,plots.TF=plots){

  ##==Purpose: To do a parametric modeling on the peaks.
  ##
  ##==Input:
  ##  -spect.id: a scalar indicating the spectrum id
  ##  -collect.tmp: a matrix containing the information from the ARS approach of obtaining peaks and valleys
  ##  -data.default: a array containing the spectra profile. The first dim is TOF, second is spectrum id,
  ##                 third c(mz,intensity).
  ##  -maxiter: the max number of iteration for the EM algorithm.
  ##  -tol: the tolerence level for which if the euclidean distance is less tol, there is convergence
  ##        It is also used to determine the smalled
  ##  -indMode: if the string is "mz", it will use mz values instead of the indices
  ##  -sOpt: the option for the parameter s. It is either fixed for all iterations or varying/updated.
  ##  -warning.path: a string indicating where the warning path is
  ##  -cluster.id: a numeric indicating the cluster id
  ##
  ##==Output: A matrix with columns corresponding to:
  ##  -peak.id: the peak id number
  ##  -sEst: the s parameter of the log-normal distribution (i.e. y > s).
  ##  -muEst: the estimate of the mu parameter of the log-normal distribution.
  ##  -sigmaEst: the estimate of the sigma parameter of the log-normal distribution.
  ##  -pEst: the estimate of the mixture probabilities.
  ##  -area.spect: the normalizing constant term for the adjusted intensity (see below)
  ##  -min.min.int: the constant term for making the intensities non-negative (i.e. adjusted intensity).
  ##  -converge: logical indicating whether EM convergence was obtained
  ##             NA means EM was not invoked because it is a one peak cluster.
  ##             NaN means EM was not invoked and the parametric approach was not able to work.
  ##             This NaN is likely due to the fact that the peak width detected via the non-parametric approach was small.
  ##             Therefore causing the estimated variance for the log-normal to be zero and thus NaN values returned as
  ##             estimates for the intensities.
  ##  -iter: the no of iteration needed to reach EM convergence
  ##
  ##==Comments:
  ##  -050107, Two new arguements Return a warning in the file and also to the output of this function
  ##  -190107, compute the (unscaled) MSE
  ##  -150307, Correction: the way 'intObsEst' is computed needed this variable to be initiated.
  ##                       Furthermore, we used the trapezium rule to compute the area for the observed intensity
  ##                       and the estimated intensity from the parametric model.
  ##  -160307, We made the ability to obtain the capture the error where the intensity returns NaN.
  ##           Therefore warning messages like below is fine:
  ##           Warning messages:
  ##               1: NaNs produced in: dlnorm(x, meanlog, sdlog, log) 
  ##           We also corrected the way to more better prediction of the intensities from parametric model
  ##           and return an additional column, area.temp in the matrix.
  ##
  ##  -06092007, Put the option for wanting the diagnostic plots, using arguement plots.TF.
  
  
      
  ##...Identifying the spectrum data that will undergo the parametric peak fitting (EM-algorithm)
  ##Identify the rows indices containing the specified spectrum (spect.id)
  validPA<-which(collect.tmp[,"spectrum"]==spect.id)
  ##Identify the mz range of the cluster of interest for the specified spectrum (spect.id)
  mz.rangePA<-range(collect.tmp[validPA,c("max.mz","left.mz","right.mz")])
  ##Identify the row ids that statisfy the mz range
  row.idPA<-which(data.default[,spect.id,1]>=mz.rangePA[1] & data.default[,spect.id,1]<=mz.rangePA[2])
  ##Identify the spectrum
  spect.raw<-data.default[row.idPA,spect.id,]
  
  
  ##...Convert the spectrum into densities
  spect<-spect.raw
  ##The min value of the intensity
  min.min.int<-min(spect[,2])
  min.min.int<-ifelse(min.min.int<=0,min.min.int,0)
  ##Make the spectrum non-negative
  spect[,2]<-spect[,2]-min.min.int
  ##Area of the spectrum
  area.spect<-0.5*sum(diff(spect[,1])*(spect[1:(nrow(spect)-1),2]+spect[2:nrow(spect),2]))
  ##Make the spectrum sum to 1
  spect[,2]<-spect[,2]/area.spect
  
    
  ##...Getting an initial estimate for the log-normal distribution as stated by the paper in Section 2.2.5
  ##The local modes/max pts
  m<-collect.tmp[validPA,"max.mz"]

  ##...Previous way of estimating m and s0 (Got extremely large values results)
  ##  s0 <- collect.tmp[validPA, "left.mz"]
  ##  m[s0 == m] <- m[s0 == m] + mean(diff(spect[, 1]))
  ##  s0[1] <- s0[1] - mean(diff(spect[, 1]))
  
  ##The local minimum values/valleys to the left of the peak
  val<-collect.tmp[validPA,"left.mz"]
  ##Substract 0.0005 to alpha0 is to prevent the first entry to be weighted zero
  alpha0<-val[1]/m[1]-0.0005
  sigma0<-0.3
  ##The suggested initial values for s; see (15)
  s0<-alpha0*m
  ##The suggested initial values for mu; see (17)
  mu0<-log(m-s0)+sigma0^2
  ##The parameters needed for shifted log normal
  par0<-cbind(s0,mu0,sigma0=rep(sigma0,length(validPA)))
  
  if(length(validPA)>1){##More than 1 peak in the cluster
    
    ##...Getting an initial estimate for the proportion, pro0 (my suggestion)
    ##Get the adjusted intensity of the mode (i.e. max.ampMix+min.int)
    intensity<-collect.tmp[validPA,"max.ampMix"]+collect.tmp[validPA,"min.int"]
    ##  and then make it non-negative using the entire cluster (i.e. min(min.int))
    intensity<-intensity-min.min.int
    ##The proportion among the modes
    pro0<-c(intensity/sum(intensity))
    
    ##Implement the EM-aglorithm
    resultPA<-EMregion(spect,par0,pro0,indMode,tol=toler,alpha=alpha0,sOpt="fix",maxiter=maxiter,
                       warning.path=warning.path,cluster.id=cluster.id,spect.id=spect.id)
    ##The convergence status and iteration from the EM algorithm
    converge<-resultPA[[3]]
    iter<-resultPA[[4]]
    
  }else{
    
    ##Implement the maximizing algorithm
    if(indMode=="mz"){
      resultPA<-Mstep(spect[,1],N=spect[,2],pji=matrix(1,nrow=nrow(spect)),
                      para0=par0,alpha=NULL)
    }else{
      resultPA<-Mstep(1:nrow(spect),N=spect[,2],pji=matrix(1,nrow=nrow(spect)),
                      para0=par0,alpha=NULL)
    }
    ##The convergence status and iteration is not applicable as EM algorithm is not applied
    converge<-NA
    iter<-NA
    
  }
  
  ##The estimated parameters
  muEst<-resultPA[[1]][,"mu"]
  sEst<-resultPA[[1]][,"s"]
  sigmaEst<-resultPA[[1]][1,"sigma"]
  pEst<-resultPA[[2]]
  
  ##...Diagnostic plot. Plot observed vs fitted.
  ##The profile from the estimated parameter
  intEst<-0
  mzR<-seq(min(spect[,1]),max(spect[,1]),length.out=3*nrow(spect))

###150307_Start
  intObsEst<-0
  intTmpArea<-NULL
###150307_End
###190106_Start
  mzObs<-spect[,1]
###190106_End
  
  for(i in 1:length(validPA)){

###190106_Start and
###150307_Start
    ##The predicted "intensities" from the parametric models at the observed points
    intTmp<-dlnorm(mzObs-sEst[i],muEst[i],sigmaEst)
    ##Area under the predicted "intensities"
    intTmpArea<-c(intTmpArea,0.5*sum(diff(mzObs)*(intTmp[1:(length(intTmp)-1)]+intTmp[2:length(intTmp)])))
    ##Sum up the components of the mixture model and normalize the "intensities"
    intObsEst<-intObsEst+(pEst[i]*intTmp)/intTmpArea[i]
###190107_End and
###150307_End
    
###150307_Start
    ##The predicted "intensities" from the parametric models at our specified points
    intTmp<-dlnorm(mzR-sEst[i],muEst[i],sigmaEst)
    ##Sum up the components of the mixture model and normalize the "intensities"
    intEst<-intEst+(pEst[i]*intTmp)/intTmpArea[i]
###150307_End and
    
  }

  ##Scale the normalized "intensities" to the actual level
  intEst<-area.spect*intEst
  
###190106_Start  
  intObsEst<-area.spect*intObsEst
  MSE<-sum((intObsEst-spect.raw[,2])^2)/length(intObsEst)
###190106_End

  ##Plot the actual data profile
  validObs<-data.default[,spect.id,1]>=min(collect.tmp[,c("max.mz","left.mz","right.mz")]) & data.default[,spect.id,1]<=max(collect.tmp[,c("max.mz","left.mz","right.mz")])
  intObs<-data.default[validObs,spect.id,]
  intObs[,2]<-intObs[,2]-min.min.int

###16032007_start:
  if(any(is.na(intObsEst))){
###==Start 06092007: Decide whether the diagnostic plot is wanted
    if(plots.TF){
      ##To plot the observed intensity
      plot(intObs,type="l",
           xlim=c(min(collect.tmp[,c("max.mz","left.mz","right.mz")]),max(collect.tmp[,c("max.mz","left.mz","right.mz")])),
           xlab="mz",ylab="intensity",main=paste("Spectrum:",spect.id),
           sub="Parametric approach did not work (i.e. converge = -1)")
    
      ##Plot the aggreagated estimated profile (red)
      lines(mzR,intEst,lty=1,col="red",lwd=2)
  
      ##Plot the estimated profile (red and dotted)
      for(i in 1:length(validPA)){
        lines(mzR,(area.spect/intTmpArea[i])*pEst[i]*dlnorm(mzR-sEst[i],muEst[i],sigmaEst),lty=2,col="red")
      }
    }
###==End 06092007: Decide whether the diagnostic plot is wanted
    converge<-NaN
    
  }else{
###16032007_end:

###==Start 06092007: Decide whether the diagnostic plot is wanted
    if(plots.TF){
      plot(intObs,type="l",ylim=range(intObs[,2],intEst),
           xlim=c(min(collect.tmp[,c("max.mz","left.mz","right.mz")]),max(collect.tmp[,c("max.mz","left.mz","right.mz")])),
           xlab="mz",ylab="intensity",main=paste("Spectrum:",spect.id))
    
      ##Plot the aggreagated estimated profile (red)
      lines(mzR,intEst,lty=1,col="red",lwd=2)
  
      ##Plot the estimated profile (red and dotted)
      for(i in 1:length(validPA)){
        lines(mzR,(area.spect/intTmpArea[i])*pEst[i]*dlnorm(mzR-sEst[i],muEst[i],sigmaEst),lty=2,col="red")
      }
    }
###==End 06092007: Decide whether the diagnostic plot is wanted
###16032007_start:    
  }
###16032007_end:


###16032007_start:  
  ##The estimates and information to output
  outPA<-cbind(peak.id=1:length(sEst),spectrum=spect.id,sEst,muEst,sigmaEst,pEst,
               area.spect,area.temp=intTmpArea,min.min.int,converge,iter,MSE)
###16032007_end:

  return(outPA)
  
}
